/***************************************************************************
                          kplwidget.cpp  -  description
                             -------------------
    begin                : Mon Jul 16 2001
    copyright            : (C) 2001 by Werner Stille
    email                : stille@uni-freiburg.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <kprinter.h>
#include "kplwidget.h"
#include "kplitem.h"

KplWidget::KplWidget(QWidget* parent) : QScrollView(parent)
{
  QScrollView::viewport()->setBackgroundMode(PaletteBase);
}

KplWidget::KplWidget(QWidget* parent, QList<KplItem>* items, int w, int h,
                     double zoom) : QScrollView(parent)
{
  QScrollView::viewport()->setBackgroundMode(PaletteBase);
  pm.resize(w, h);
  resizeContents(w, h);
  plInit(&pm);
  factor = zoom;
  drawItems(this, items);
  end();
}

KplWidget::~KplWidget()
{
}

void KplWidget::drawItems(KplGraph* g, QList<KplItem>* items)
{
  for (KplItem* item = items->first(); item; item = items->next())
    item->draw(g);
}

void KplWidget::print(KPrinter* printer, QList<KplItem>* items)
{
  KGraph g;
  QSize sz = printer->margins();
  g.plInit(printer, sz.width(), sz.height());
  drawItems(&g, items);
  g.end();
}

QSize KplWidget::sizeHint() const
{
  return QScrollView::sizeHint() + QSize(1, 1);
}

void KplWidget::drawContentsOffset(QPainter*, int offsetx, int offsety,
                                   int clipx, int clipy, int clipw, int cliph)
{
  int x1 = QMAX(0, clipx) - offsetx;
  int y1 = QMAX(0, clipy) - offsety;
  int x2 = QMIN(pm.width(), clipx + clipw - 1) - offsetx;
  int y2 = QMIN(pm.height(), clipy + cliph - 1) - offsety;
  if (x2 >= x1 && y2 >= y1)
    bitBlt(QScrollView::viewport(), x1, y1, &pm, clipx, clipy, clipw, cliph,
           CopyROP, true);
}
